@@ -1158,6 +1158,7 @@ tcaphash_cont_matching(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   proto_item *pi;
   proto_item *stat_item=NULL;
   proto_tree *stat_tree=NULL;
+  gboolean use_dst = TRUE;
 
 #ifdef DEBUG_TCAPSRT
   dbg(51,"src %s srcTid %lx dst %s dstTid %lx ", address_to_str(wmem_packet_scope(), &pinfo->src), p_tcapsrt_info->src_tid, address_to_str(wmem_packet_scope(), &pinfo->dst), p_tcapsrt_info->dst_tid);
@@ -1213,11 +1214,13 @@ tcaphash_cont_matching(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 #endif
     p_tcaphash_begincall = find_tcaphash_begin(&tcaphash_begin_key, pinfo, FALSE);
     if(!p_tcaphash_begincall){
+      try_src:
 /* can this actually happen? */
 #ifdef DEBUG_TCAPSRT
         dbg(12,"BNotFound trying stid,src");
 #endif
         /* Do we have a continue from the same source? (stid,src) */
+        use_dst = FALSE;
         tcaphash_begin_key.tid = p_tcapsrt_info->src_tid;
         if (pinfo->src.type == AT_SS7PC && pinfo->dst.type == AT_SS7PC)
         {
@@ -1250,21 +1253,21 @@ tcaphash_cont_matching(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
       create_tcaphash_cont(&tcaphash_cont_key,
                            p_tcaphash_begincall->context);
 
-      /* Create END for (stid,src) */
-      tcaphash_end_key.tid = p_tcapsrt_info->src_tid;
+      /* Create END for (dtid,dst) or (stid,src) */
+      tcaphash_end_key.tid = use_dst ? p_tcapsrt_info->dst_tid : p_tcapsrt_info->src_tid;
       if (pinfo->src.type == AT_SS7PC && pinfo->dst.type == AT_SS7PC)
       {
         /* We have MTP3 PCs (so we can safely do this cast) */
-        tcaphash_end_key.pc_hash = mtp3_pc_hash((const mtp3_addr_pc_t *)pinfo->src.data);
+        tcaphash_end_key.pc_hash = mtp3_pc_hash((const mtp3_addr_pc_t *)(use_dst ? pinfo->dst.data : pinfo->src.data));
       } else {
         /* Don't have MTP3 PCs (have SCCP GT ?) */
-        tcaphash_end_key.pc_hash = g_str_hash(address_to_str(wmem_packet_scope(), &pinfo->src));
+        tcaphash_end_key.pc_hash = g_str_hash(address_to_str(wmem_packet_scope(), use_dst ? &pinfo->dst : &pinfo->src));
       }
       tcaphash_end_key.hashKey=tcaphash_end_calchash(&tcaphash_end_key);
 
 #ifdef DEBUG_TCAPSRT
       dbg(10,"New Ekey %lx ",tcaphash_end_key.hashKey);
-      dbg(51,"addr %s ", address_to_str(wmem_packet_scope(), &pinfo->src));
+      dbg(51,"addr %s ", address_to_str(wmem_packet_scope(), use_dst ? &pinfo->dst : &pinfo->src));
       dbg(51,"Tid %lx ",tcaphash_end_key.tid);
       dbg(11,"Frame reqlink #%u ", pinfo->fd->num);
 #endif
@@ -1275,6 +1278,10 @@ tcaphash_cont_matching(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 #ifdef DEBUG_TCAPSRT
       dbg(12,"BnotFound ");
 #endif
+      if (use_dst) {
+        /* make another try with src tid / address */
+        goto try_src;
+      }
     } /* begin found */
   } /* cont found */
     /* display tcap session, if available */
@@ -2071,6 +2078,7 @@ proto_reg_handoff_tcap(void)
 }
 
 static void init_tcap(void);
+static void cleanup_tcap(void);
 
 void
 proto_register_tcap(void)
@@ -2187,7 +2195,6 @@ proto_register_tcap(void)
 
   /* Set default SSNs */
   range_convert_str(&global_ssn_range, "", MAX_SSN);
-  ssn_range = range_empty();
 
   prefs_register_range_preference(tcap_module, "ssn", "SCCP SSNs",
                                   "SCCP (and SUA) SSNs to decode as TCAP",
@@ -2222,6 +2229,7 @@ proto_register_tcap(void)
   tcap_handle = create_dissector_handle(dissect_tcap, proto_tcap);
 
   register_init_routine(&init_tcap);
+  register_cleanup_routine(&cleanup_tcap);
 }
 
 
@@ -2242,16 +2250,17 @@ static void range_add_callback(guint32 ssn)
 
 static void init_tcap(void)
 {
-  if (ssn_range) {
-    range_foreach(ssn_range, range_delete_callback);
-    g_free(ssn_range);
-  }
-
   ssn_range = range_copy(global_ssn_range);
   range_foreach(ssn_range, range_add_callback);
   tcapsrt_init_routine();
 }
 
+static void cleanup_tcap(void)
+{
+  range_foreach(ssn_range, range_delete_callback);
+  g_free(ssn_range);
+}
+
 static int
 dissect_tcap_param(asn1_ctx_t *actx, proto_tree *tree, tvbuff_t *tvb, int offset)
 {
