@@ -61,6 +61,7 @@ static int hf_smtp_rsp_code = -1;
 static int hf_smtp_rsp_parameter = -1;
 static int hf_smtp_username = -1;
 static int hf_smtp_password = -1;
+static int hf_smtp_username_password = -1;
 static int hf_smtp_eom = -1;
 
 static int hf_smtp_data_fragments = -1;
@@ -80,7 +81,9 @@ static int ett_smtp_cmdresp = -1;
 static gint ett_smtp_data_fragment = -1;
 static gint ett_smtp_data_fragments = -1;
 
-static gboolean    smtp_decryption_enabled     = FALSE;
+static expert_field ei_smtp_base64_decode = EI_INIT;
+
+static gboolean    smtp_auth_parameter_decoding_enabled     = FALSE;
 /* desegmentation of SMTP command and response lines */
 static gboolean    smtp_desegment              = TRUE;
 static gboolean    smtp_data_desegment         = TRUE;
@@ -314,9 +317,10 @@ decode_plain_auth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   gint                       length_user2;
   gint                       length_pass;
   guint8                    *decrypt   = NULL;
+  proto_item                *ti;
 
   decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, a_offset, a_linelen, ENC_ASCII);
-  if (smtp_decryption_enabled) {
+  if (smtp_auth_parameter_decoding_enabled) {
     returncode = (gint)ws_base64_decode_inplace(decrypt);
     if (returncode) {
       length_user1 = (gint)strlen(decrypt);
@@ -339,10 +343,9 @@ decode_plain_auth(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     }
   }
   else {
-    proto_tree_add_string(tree, hf_smtp_username, tvb,
-                          a_offset, a_linelen, decrypt);
-    proto_tree_add_string(tree, hf_smtp_password, tvb,
-                          a_offset, a_linelen, decrypt);
+    ti = proto_tree_add_item(tree, hf_smtp_username_password, tvb,
+                          a_offset, a_linelen, ENC_ASCII|ENC_NA);
+    expert_add_info(pinfo, ti, &ei_smtp_base64_decode);
     col_append_str(pinfo->cinfo, COL_INFO, format_text(decrypt, a_linelen));
   }
 }
@@ -522,6 +525,10 @@ dissect_smtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                * Check if we have reached end of the data chunk.
                */
               session_state->msg_read_len += tvb_reported_length_remaining(tvb, loffset);
+              /*
+               * Since we're grabbing the rest of the packet, update the offset accordingly
+               */
+              next_offset = tvb_reported_length(tvb);
 
               if (session_state->msg_read_len == session_state->msg_tot_len) {
                 /*
@@ -555,7 +562,7 @@ dissect_smtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
               (pinfo->fd->num >= session_state->first_auth_frame) &&
               ((session_state->last_auth_frame == 0) || (pinfo->fd->num <= session_state->last_auth_frame))) {
             decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset, linelen, ENC_ASCII);
-            if ((smtp_decryption_enabled) &&
+            if ((smtp_auth_parameter_decoding_enabled) &&
                 ((decrypt_len = ws_base64_decode_inplace(decrypt)) > 0)) {
               line = decrypt;
               linelen = (int)decrypt_len;
@@ -817,7 +824,7 @@ dissect_smtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             /* This line wasn't already decrypted through the state machine */
             decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset, linelen, ENC_ASCII);
             decrypt_len = linelen;
-            if (smtp_decryption_enabled) {
+            if (smtp_auth_parameter_decoding_enabled) {
               if ((decrypt_len = ws_base64_decode_inplace(decrypt)) == 0) {
                 /* Go back to the original string */
                 decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset, linelen, ENC_ASCII);
@@ -833,7 +840,7 @@ dissect_smtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             /* This line wasn't already decrypted through the state machine */
             decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset, linelen, ENC_ASCII);
             decrypt_len = linelen;
-            if (smtp_decryption_enabled) {
+            if (smtp_auth_parameter_decoding_enabled) {
               if ((decrypt_len = ws_base64_decode_inplace(decrypt)) == 0) {
                 /* Go back to the original string */
                 decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset, linelen, ENC_ASCII);
@@ -847,7 +854,7 @@ dissect_smtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         } else if (session_state->ntlm_rsp_frame == pinfo->fd->num) {
           decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset, linelen, ENC_ASCII);
           decrypt_len = linelen;
-          if (smtp_decryption_enabled) {
+          if (smtp_auth_parameter_decoding_enabled) {
             if ((decrypt_len = ws_base64_decode_inplace(decrypt)) == 0) {
               /* Go back to the original string */
               decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset, linelen, ENC_ASCII);
@@ -894,7 +901,7 @@ dissect_smtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 /* This line wasn't already decrypted through the state machine */
                  decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset + 11, linelen - 11, ENC_ASCII);
                  decrypt_len = linelen - 11;
-                 if (smtp_decryption_enabled) {
+                 if (smtp_auth_parameter_decoding_enabled) {
                    if ((decrypt_len = ws_base64_decode_inplace(decrypt)) == 0) {
                      /* Go back to the original string */
                      decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset + 11, linelen - 11, ENC_ASCII);
@@ -914,7 +921,7 @@ dissect_smtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             if (linelen >= 10) {
               decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset + 10, linelen - 10, ENC_ASCII);
               decrypt_len = linelen - 10;
-              if (smtp_decryption_enabled) {
+              if (smtp_auth_parameter_decoding_enabled) {
                 if ((decrypt_len = ws_base64_decode_inplace(decrypt)) == 0) {
                   /* Go back to the original string */
                   decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, loffset + 10, linelen - 10, ENC_ASCII);
@@ -1096,7 +1103,7 @@ dissect_smtp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
             decrypt = NULL;
             if (linelen >= 4) {
-                if ((smtp_decryption_enabled) && (code == 334)) {
+                if ((smtp_auth_parameter_decoding_enabled) && (code == 334)) {
                     decrypt = tvb_get_string_enc(wmem_packet_scope(), tvb, offset + 4, linelen - 4, ENC_ASCII);
                     if ((decrypt_len = ws_base64_decode_inplace(decrypt)) > 0) {
                       if (g_ascii_strncasecmp(decrypt, "NTLMSSP", 7) == 0) {
@@ -1146,6 +1153,12 @@ smtp_data_reassemble_init (void)
                         &addresses_ports_reassembly_table_functions);
 }
 
+static void
+smtp_data_reassemble_cleanup (void)
+{
+  reassembly_table_destroy(&smtp_data_reassembly_table);
+}
+
 
 /* Register all the bits needed by the filtering engine */
 
@@ -1197,6 +1210,10 @@ proto_register_smtp(void)
       { "Password", "smtp.auth.password",
         FT_STRING, BASE_NONE, NULL, 0x0, NULL, HFILL }},
 
+    { &hf_smtp_username_password,
+      { "Username/Password", "smtp.auth.username_password",
+        FT_STRING, BASE_NONE, NULL, 0x0, NULL, HFILL }},
+
     { &hf_smtp_eom,
       { "EOM", "smtp.eom",
         FT_NONE, BASE_NONE, NULL, 0x00, NULL, HFILL } },
@@ -1250,14 +1267,23 @@ proto_register_smtp(void)
     &ett_smtp_data_fragments,
 
   };
+
+  static ei_register_info ei[] = {
+    { &ei_smtp_base64_decode, { "smtp.base64_decode", PI_PROTOCOL, PI_WARN, "base64 decode failed or is not enabled (check SMTP preferences)", EXPFILL }},
+  };
+
   module_t *smtp_module;
+  expert_module_t* expert_smtp;
 
   proto_smtp = proto_register_protocol("Simple Mail Transfer Protocol",
                                        "SMTP", "smtp");
 
   proto_register_field_array(proto_smtp, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
+  expert_smtp = expert_register_protocol(proto_smtp);
+  expert_register_field_array(expert_smtp, ei, array_length(ei));
   register_init_routine (&smtp_data_reassemble_init);
+  register_cleanup_routine (&smtp_data_reassemble_cleanup);
 
   /* Allow dissector to find be found by name. */
   register_dissector("smtp", dissect_smtp, proto_smtp);
@@ -1279,9 +1305,9 @@ proto_register_smtp(void)
                                  &smtp_data_desegment);
 
   prefs_register_bool_preference(smtp_module, "decryption",
-                                 "Decrypt AUTH parameters",
-                                 "Whether the SMTP dissector should decrypt AUTH parameters",
-                                 &smtp_decryption_enabled);
+                                 "Decode base-64-encoded AUTH parameters",
+                                 "Whether the SMTP dissector should decode base-64-encoded AUTH parameters",
+                                 &smtp_auth_parameter_decoding_enabled);
 }
 
 /* The registration hand-off routine */
